﻿using System.Collections;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Persistence.Interfaces;
using System;

namespace HeuristicLab.Persistence {
  

  public abstract class NumberEnumeration2XmlFormatter : IFormatter {

    public abstract Type Type { get; }
    public IFormat Format { get { return XmlFormat.Instance; } }
    protected virtual string Separator { get { return ";"; } }
    protected abstract void Add(IEnumerable enumeration, object o);
    protected abstract object Instantiate();
    protected abstract string formatValue(object o);
    protected abstract object parseValue(string o);

    public object Serialize(object o) {
      StringBuilder sb = new StringBuilder();
      foreach (var value in (IEnumerable)o) {
        sb.Append(formatValue(value));
        sb.Append(Separator);
      }
      return sb.ToString();
    }

    public object DeSerialize(object o) {
      IEnumerable enumeration = (IEnumerable)Instantiate();
      string[] values = ((string)o).Split(new[] { Separator }, StringSplitOptions.RemoveEmptyEntries);
      foreach (var value in values) {
        Add(enumeration, parseValue(value));
      }
      return enumeration;
    }
  }


  public class IntList2XmlFormatter : NumberEnumeration2XmlFormatter {
    public override Type Type { get { return typeof(List<int>); } }
    protected override void Add(IEnumerable enumeration, object o) { ((List<int>)enumeration).Add((int)o); }
    protected override object Instantiate() { return new List<int>(); }
    protected override string formatValue(object o) { return o.ToString(); }
    protected override object parseValue(string o) { return int.Parse(o); }
  }


  public class DoubleList2XmlFormatter : NumberEnumeration2XmlFormatter {
    public override Type Type { get { return typeof(List<double>); } }
    protected override void Add(IEnumerable enumeration, object o) { ((List<double>)enumeration).Add((int)o); }
    protected override object Instantiate() { return new List<double>(); }
    protected override string formatValue(object o) { return ((double)o).ToString("r"); }
    protected override object parseValue(string o) { return double.Parse(o); }
  }
}